pub mod metar_tab;

use std::collections::HashMap;

// Strum Package to make enum Itterable
use strum::EnumIter;
use strum::IntoEnumIterator;

use crate::App;

#[derive(Debug, Default, EnumIter, PartialEq, Eq, Hash)]
pub enum TabTypes {
    #[default]
    Home,
    METAR,
    TAF,
    NOTAMS,
    Routing,
}

#[derive(Default)]
pub struct TabState {
    current_tab: TabTypes,
    tabs: HashMap<TabTypes, Box<dyn Tab>>,
}

impl TabState {
    /// Get the Tab based on a given Tab Type
    pub fn get_tab_state(&self, tab_type: &TabTypes) -> Option<&Box<dyn Tab>> {
        match self.tabs.get(tab_type) {
            Some(tab) => Some(tab),
            None => None,
        }
    }

    /// Set the state of a given tab
    pub fn set_tab_state(&mut self, tab_type: &TabTypes) {}

    /// Get the currently selected Tab
    pub fn get_current_tab(&self) -> Option<&Box<dyn Tab>> {
        self.get_tab_state(&self.current_tab)
    }

    /// Get the type of the current Tab
    pub fn get_current_tab_type(&self) -> &TabTypes {
        return &self.current_tab;
    }

    /// Change the current tab to a different tab type, saving the state of the current tab
    pub fn change_tab(&mut self, new_tab_type: &TabTypes) {
        self.current_tab = *new_tab_type;
    }
}

/// Tab traits that allow tab display on the main application window
pub trait Tab {
    fn display_tab(&mut self, ctx: &egui::Context);
}

/// Function to draw main set of tabs on the window
pub fn display_tabs(app: &mut App, ctx: &egui::Context) {
    egui::TopBottomPanel::top("app_top_bar").show(ctx, |ui| {
        ui.horizontal_wrapped(|ui| {
            ui.visuals_mut().button_frame = false;

            // Insert Tabs
            for tab in TabTypes::iter() {
                let tab_label =
                    ui.selectable_label(tab == app.tab_state.current_tab, format!("{:?}", tab));

                if tab_label.clicked() {
                    app.tab_state.current_tab = tab
                }
            }
        });
    });
}
